;================================================================
;
;  IR Receiver/Sender/Repeater
;  version 1.0
;  
;  AVR Studio   /www.atmel.com/
;
;================================================================

.nolist
.include "tn2313def.inc"
.list


.def iareg = r15
.def areg = r16
.def breg = r17
.def ir_lo = r18
.def ir_hi = r19
.def usartptr = r20
.def usarttimeout = r21
.def ir_flag0 = r22
.def ir_flag1 = r23
.def ir_idx = r24
.def xareg = r25
.def delkhz = r26
; X=r26,r27, Y=r28,r29, Z=r30,r31

; flag0
.equ ir_30kHz = 0
.equ ir_33kHz = 1
.equ ir_36kHz = 2 
.equ ir_37kHz = 3   ; 36,7kHz
.equ ir_38kHz = 4
.equ ir_40kHz = 5
.equ ir_56kHz = 6

; flag1
.equ ir_repeat = 0
.equ ir_sw_rep = 1
.equ ir_sndLoHi = 2
.equ ir_sndHiLo = 3
.equ ir_kHz = 4
.equ ir_det_Hz = 6
.equ ir_irda_rep = 7

.equ usarttimeoutconst = 16 ; 0,76s

.equ Timer0Prescaler_kHz = 0b00000001 ; min=0,0000000904224537037 
.equ Timer0Prescaler = 0b00000100     ; min=0,0000231481  max=0,00592592
.equ Timer1Prescaler = 0b00000010



.equ DirPortA = 0b00000000

.equ Pix_Repeater = PINB2
.equ Pin_Repeater = PINB
.equ Pix_IRDAIn   = PINB3
.equ Pin_IRDAIn   = PINB

.equ Pix_IRDAOut  = PINB6
.equ Pin_IRDAOut  = PINB
.equ Por_IRDAOut  = PORTB
.equ Pix_DS18B20  = PINB7
.equ Pin_DS18B20  = PINB
.equ Por_DS18B20  = PORTB
.equ DirPortB = 0b01100000

.equ Pix_IROut    = PIND6
.equ Pin_IROut    = PIND
.equ Por_IROut    = PORTD
.equ DirPortD = 0b01000010

.equ MaxUartChars = 10
.equ CmdLength = 10
.equ MaxIrIdx = 20
.equ IRBitCodeLen = 9
.equ IRDataCodeLen = 4
.equ EpromIRBitCode = 8
.equ EpromIRDataCode00 = 24

.eseg  ; eeprom 128 bytes

.db 16,0,0,0,0,0,0,0 ; flag0 = 38kHz, flag1 = 0
.db 2,172,170,22,1,1,22,20
.db 62,0,0,0,0,0,0,0
.db 0x40, 0xBF, 0x30, 0xCF

.dseg  ; SRAM 128 bytes

ds18data: .byte 9
ds18crc: .byte 1
ds18err: .byte 1
 
usartdata: .byte MaxUartChars
ir_dt_data: .byte 2*MaxIrIdx
ir_DataCode: .byte IRDataCodeLen
ir_BitCode: .byte IRBitCodeLen

.cseg

   .org $0000
   rjmp Main 
   .org $0001
   rjmp int0rq
   .org $0002
   rjmp int1rq
   .org $0003
   reti
   .org $0004
   reti
   .org $0005
   rjmp timer1
   .org $0006
   rjmp timer0
   .org $0007
   rjmp UsartRx
   .org $0008
   reti
   .org $0009
   reti
   .org $000A
   reti
   .org $000B
   rjmp PinChangeInt
   .org $000C
   reti
   .org $000D
   reti
   .org $000E
   reti
   .org $000F
   reti
   .org $0010
   reti
   .org $0011
   reti
   .org $0012
   rjmp wdtovf



ClearTimer0:
   in    ir_lo,tcnt0
   ldi   areg,0
   out   tccr0b,areg
   out   tcnt0,areg
   ldi   areg,Timer0Prescaler 
   out   tccr0b,areg
   ret

IntX:
   mov   areg,ir_hi
   rcall SendChar
   mov   areg,ir_lo  
   rcall SendChar
IntX1:
   clr   ir_hi
   pop   areg
   out   sreg,areg   
   pop   areg
   reti

Int0rq:
   push  areg
   in    areg,sreg    
   push  areg

   rcall ClearTimer0
   sbr   ir_flag1,(1 << ir_khz)
   sbrc  ir_flag1,ir_sndHiLo
   rjmp  IntX
   rjmp  IntX1

Int1rq:
   push  areg
   in    areg,sreg    
   push  areg

   rcall ClearTimer0
   cbr   ir_flag1,(1 << ir_khz)
   sbrc  ir_flag1,ir_sndLoHi   
   rjmp  IntX
   rjmp  IntX1

Timer0:   
   in    iareg,sreg    
   cpi   ir_hi,255
   breq  Timer0_0
   inc   ir_hi
Timer0_0:
   out   sreg,iareg   
   reti		


Timer1:
   push  areg
   in    areg,sreg    
   push  areg
Timer1_0:
   tst   usarttimeout
   breq  Timer1_1
   dec   usarttimeout
   rjmp  Timer1ex
Timer1_1:
   tst   usartptr
   breq  Timer1ex
   clr   usartptr
Timer1ex:
   pop   areg
   out   sreg,areg   
   pop   areg
   reti

PinChangeInt:
   sbic  Pin_IRDAIn,Pix_IRDAIn
   rjmp  PinChangeInt2
   in    iareg,sreg    
   in    ir_lo,tcnt0
   ldi   xareg,0
   out   tccr0b,xareg
   out   tcnt0,xareg
   ldi   xareg,Timer0Prescaler_kHz
   out   tccr0b,xareg
   sbrc  ir_flag1,ir_irda_rep
   sbi   Por_IROut,Pix_IROut 
   cpi   ir_hi,255
   brne  PinChangeInt0
   rjmp  PinChangeInt1
PinChangeInt0:
   cpi   ir_idx,MaxIrIdx
   brlo  PinChangeInt3
   clr   ir_idx
PinChangeInt3:
   push  zl
   push  zh
   ldi   zh,high(ir_dt_data)
   ldi   zl,low(ir_dt_data)
   mov   xareg,ir_idx
   lsl   xareg
   add   zl,xareg
   st    z+,ir_hi
   st    z,ir_lo
   pop   zh
   pop   zl
   inc   ir_idx
PinChangeInt1:
   clr   ir_hi
   out   sreg,iareg   
PinChangeInt2:
   sbrs  ir_flag1,ir_irda_rep
   reti
   sbic  Pin_IRDAIn,Pix_IRDAIn
   cbi   Por_IROut,Pix_IROut
   reti

UsartRx:
   ldi   usarttimeout,usarttimeoutconst  
   push  areg
   in    areg,sreg    
   push  areg
   push  zh
   push  zl
   in    areg,udr
   cpi   usartptr,MaxUartChars
   brcc  UsartRxEx
   ldi   zh,high(usartdata)
   ldi   zl,low(usartdata)
   add   zl,usartptr
;   clr   xreg
;   adc zh,xreg
   st    z,areg
   inc   usartptr
UsartRxEx:
   pop   zl
   pop   zh
   pop   areg
   out   sreg,areg   
   pop   areg
   reti

wdtovf:
   reti



Main:  
   ldi   areg,ramend
   out   spl,areg
   ldi   areg,DirPortA
   out   ddra,areg
   ldi   areg,DirPortB
   out   ddrb,areg
   ldi   areg,DirPortD
   out   ddrd,areg
   ldi   areg,0b00000000
   out   porta,areg
   ldi   areg,0b00011100
   out   portb,areg
   ldi   areg,0b00001101
   out   portd,areg
   clr   ir_idx
   clr   usartptr
   clr   ir_flag1
   clr   areg
   rcall LoadEEPROM
   mov   ir_flag0,areg
   ldi   areg,1
   rcall LoadEEPROM
   mov   ir_flag1,areg
   rcall ClearIRDtc
   ldi   delkHz,27            ; default 38kHz
   rcall InitUSART
   ldi   areg,0
   out   tccr0a,areg
   out   tccr0b,areg
   ldi   areg,Timer0Prescaler 
   out   tccr0b,areg 
   ldi   areg,0
   out   tccr1a,areg
   out   tccr1b,areg
   ldi   areg,Timer1Prescaler 
   out   tccr1b,areg
   ldi   areg,0b10000010
   out   timsk,areg
   in    areg,mcucr
   ori   areg,0b00001110
   out   mcucr,areg
   ldi   areg,(1 << Pix_IRDAIn)
   out   pcmsk,areg 
   rcall CheckInt
   rcall Delay100ms
   sei

LOOP:
   wdr
   cpi   usartptr,CmdLength
   brcc  DispCmd

   sbis  Pin_Repeater,Pix_Repeater
   sbr   ir_flag1,(1 << ir_repeat)     ; request is set by hardware
   sbic  Pin_Repeater,Pix_Repeater
   cbr   ir_flag1,(1 << ir_repeat)
   sbrc  ir_flag1,ir_sw_rep            ; request is set by software
   sbr   ir_flag1,(1 << ir_repeat)

   sbrc  ir_flag1,ir_irda_rep
   rjmp  LOOP

   sbrs  ir_flag1,ir_repeat
   rjmp  ContLoop
   sbrs  ir_flag1,ir_khz 
   rjmp  ContLoop

   sbi   Por_IROut,Pix_IROut
   rcall LoopDelay_kHz
   cbi   Por_IROut,Pix_IROut
   rcall LoopDelay_kHz

   rjmp  LOOP
ContLoop:
   cbi   Por_IROut,Pix_IROut
   rjmp  LOOP


DispCmd:
   cbi   ucsrb,RXEN
   lds   areg,usartdata
   mov   breg,usartptr
   clr   usartptr
DispCmd0:
   cpi   areg,0
   brne  DispCmd1
   rcall SendCommand
   rjmp  DispCmdE
DispCmd1:
   cpi   areg,1
   brne  DispCmd2
   rcall SendVersion
   rjmp  DispCmdE
DispCmd2:
   cpi   areg,2
   brne  DispCmd3

   push  ir_flag0   
   push  ir_flag1
   lds   ir_flag0,usartdata+1 
   lds   ir_flag1,usartdata+2
   rcall CheckInt
   pop   areg
   cp    areg,ir_flag1
   pop   areg
   brne  DispCmd2w
   cp    areg,ir_flag0
   brne  DispCmd2w
   rjmp  DispCmd2x
DispCmd2w:
   clr   areg
   mov   breg,ir_flag0
   rcall StoreEEPROM
   ldi   areg,1
   mov   breg,ir_flag1
   andi  breg, 255 ^ (1 << ir_sw_rep) ^ (1 << ir_repeat) ^ (1 << ir_kHz)
   rcall StoreEEPROM
   ldi   areg,'S'
   rcall SendChar
   ldi   areg,'T'
   rcall SendChar
DispCmd2x:
   rjmp  DispCmdE
DispCmd3:   
   cpi   areg,3
   brne  DispCmd4

   rcall SendInfo
   mov   areg,ir_flag0
   rcall SendChar
   mov   areg,ir_flag1
   rcall SendChar
   rcall SendIrDtc

   rjmp DispCmdE
DispCmd4:
   cpi areg,4
   brne DispCmd5

   rcall LoadIRBitCode
 
   rjmp DispCmdE
DispCmd5:
   cpi areg,5
   brne DispCmd6

   rcall LoadIRDataCode
   rcall SendIRCode

   rjmp DispCmdE
DispCmd6:
   cpi areg,6
   brne DispCmd7
   rcall SendCRLF
   cli
   wdr
   ldi areg,(1 << WDE) | (1 << WDCE)
   out wdtcsr,areg
Restart:
   rjmp Restart
   rjmp DispCmdE
DispCmd7:
   cpi areg,7
   brne DispCmd8

   rcall ReadEprom

   rjmp DispCmdE
DispCmd8:
   cpi areg,8
   brne DispCmd9

   rcall WriteEprom
  
   rjmp DispCmdE
DispCmd9:
   cpi areg,9
   brne DispCmd10

   rcall SendEpromIRCode

   rjmp DispCmdE
DispCmd10:
   cpi areg,10
   brne DispCmd11

   rcall Ds18Measure
   rcall Ds18SendData

   rjmp DispCmdE
DispCmd11:
   cpi areg,11
   brne DispCmd12

   rcall Ds18WriteCfg

   rjmp DispCmdE
DispCmd12:
   cpi areg,12
   brne DispCmd13

   rcall Ds18ReadRom
   rcall Ds18SendData

   rjmp DispCmdE
DispCmd13:
   cpi areg,13
   brne DispCmd14

   rcall SendPmIRCode

   rjmp DispCmdE
DispCmd14:
   ldi areg,'N'
   rcall SendChar
   ldi areg,'I'
   rcall SendChar

DispCmdE:
   rcall SendCRLF
   clr usartptr
   sbi ucsrb,RXEN
   rjmp LOOP


CheckInt:
   ldi   areg,(1 << INT0) | (1 << INT1)
   sbrc  ir_flag1,ir_det_Hz
   ldi   areg,(1 << PCIE)
   out   gimsk,areg    
   cbr   ir_flag1,(1 << ir_khz)
   cbi   Por_IROut,Pix_IROut
   ret

DelayXkHz:
   push  areg
   mov   areg,delkhz
DelayXkHz1:
   wdr
   dec   areg   
   brne  DelayXkHz1
   pop   areg       
   ret

LoopDelay_kHz:
   sbrc  ir_flag0,ir_30kHz
   ldi   delkHz,35
   sbrc  ir_flag0,ir_33kHz
   ldi   delkHz,31
   sbrc  ir_flag0,ir_36kHz
   ldi   delkHz,28
   sbrc  ir_flag0,ir_37kHz
   ldi   delkHz,27
   sbrc  ir_flag0,ir_38kHz
   ldi   delkHz,26
   sbrc  ir_flag0,ir_40kHz
   ldi   delkHz,24
   sbrc  ir_flag0,ir_56kHz
   ldi   delkHz,14
   rcall DelayXkHz
   ret

Delay_kHz:
   sbrc  ir_flag0,ir_30kHz
   ldi   delkHz,37
   sbrc  ir_flag0,ir_33kHz
   ldi   delkHz,33
   sbrc  ir_flag0,ir_36kHz
   ldi   delkHz,30
   sbrc  ir_flag0,ir_37kHz
   ldi   delkHz,29
   sbrc  ir_flag0,ir_38kHz
   ldi   delkHz,27
   sbrc  ir_flag0,ir_40kHz
   ldi   delkHz,26
   sbrc  ir_flag0,ir_56kHz
   ldi   delkHz,16
   rcall DelayXkHz
   ret

LoadIRBitCode:
   ldi zh,high(ir_BitCode)
   ldi zl,low(ir_BitCode)
   ldi yh,high(usartdata+1)
   ldi yl,low(usartdata+1)
   ldi breg,IRBitCodeLen
LoadData0:
   ld areg,y+
   st z+,areg
   dec breg
   brne LoadData0
   ret

LoadIRDataCode:
   ldi zh,high(ir_DataCode)
   ldi zl,low(ir_DataCode)
   ldi yh,high(usartdata+1)
   ldi yl,low(usartdata+1)
   ldi breg,IRDataCodeLen
LoadData1:
   ld areg,y+
   st z+,areg
   dec breg
   brne LoadData1
   ret

SendkHz:
   sbi   Por_IROut,Pix_IROut
   rcall Delay_kHz
   cbi   Por_IROut,Pix_IROut
   rcall Delay_kHz
   dec   areg
   brne  SendkHz
   ret

NoSendkHz:
   cbi   Por_IROut,Pix_IROut
   rcall Delay_kHz
   cbi   Por_IROut,Pix_IROut
   rcall Delay_kHz
   dec   areg
   brne  NoSendkHz
   ret

SendIRCode:
   in    areg,gimsk
   push  areg
   clr   areg
   out   gimsk,areg

   ldi zh,high(ir_BitCode)
   ldi zl,low(ir_BitCode)
   ldi yh,high(ir_DataCode)
   ldi yl,low(ir_DataCode)

   ld    breg,z
SendIRCode0:
   ldd   areg,z+1
   rcall SendkHz
   dec   breg
   brne  SendIRCode0
   ldd   areg,z+2
   rcall NoSendkHz

   ldi   breg,IRDataCodeLen
SendIRCode1:
   push  breg
   ldi   breg,8
   ld    areg,y+
SendIRCode2:
   push  areg
   ldd   areg,z+6
   rcall SendkHz
   pop   areg
   rol   areg
   push  areg
   brcs  SendIRCode3
   ldd   areg,z+7
   rjmp  SendIRCode4
SendIRCode3:
   ldd   areg,z+8
SendIRCode4:
   rcall NoSendkHz

   pop  areg
   dec  breg
   brne SendIRCode2
   pop  breg
   dec  breg
   brne SendIRCode1

   ldd   areg,z+3
   rcall SendkHz
   ldd    breg,z+4
SendIRCode5:
   ldd   areg,z+5
   rcall NoSendkHz
   dec   breg
   brne  SendIRCode5

   pop   areg
   out   gimsk,areg
   ret


SendEpromIRCode:
   ldi yh,high(ir_BitCode)
   ldi yl,low(ir_BitCode)
   ldi zl,EpromIRBitCode
   ldi breg,IRBitCodeLen
SendEpromIRCode0:
   mov   areg,zl
   rcall LoadEEPROM
   st  y+,areg
   inc zl
   dec breg
   brne SendEpromIRCode0

   ldi yh,high(ir_DataCode)
   ldi yl,low(ir_DataCode)
   ldi zl,EpromIRDataCode00
   ldi breg,IRDataCodeLen
SendEpromIRCode1:
   mov   areg,zl
   rcall LoadEEPROM
   st  y+,areg
   inc zl
   dec breg
   brne SendEpromIRCode1

   rcall SendIRCode
   ret

;               SLH  SLL   SH   EL   EHH  EHL   BL   B0   B1
;===========================================================
IRBitCode: .db    2, 172, 170,  22,   1,   1,   22,  20,  62             
IRDataCode00: .db 0x40, 0xBF, 0x30, 0xCF
;===========================================================

SendPmIRCode:
   ldi zh,high(IRBitCode << 1)
   ldi zl,low(IRBitCode << 1)
   ldi yh,high(ir_BitCode)
   ldi yl,low(ir_BitCode)
   ldi breg,IRBitCodeLen
SendPmIRCode0:
   lpm areg,z+
   st  y+,areg
   dec breg
   brne SendPmIRCode0

   ldi zh,high(IRDataCode00 << 1)
   ldi zl,low(IRDataCode00 << 1)
   ldi yh,high(ir_DataCode)
   ldi yl,low(ir_DataCode)
   ldi breg,IRDataCodeLen
SendPmIRCode1:
   lpm areg,z+
   st  y+,areg
   dec breg
   brne SendPmIRCode1

   rcall SendIRCode
   ret

ClearIRDtc:
   ldi   breg,2*MaxIrIdx
   ldi   zh,high(ir_dt_data)
   ldi   zl,low(ir_dt_data)
   clr   areg
ClearIRDtc1:
   st    z+,areg
   dec   breg
   brne  ClearIRDtc1
   ret

SendIRDtc:
   ldi   breg,2*MaxIrIdx
   ldi   zh,high(ir_dt_data)
   ldi   zl,low(ir_dt_data)
SendIRDtc1:
   ld    areg,z+
   rcall SendChar
   dec   breg
   brne  SendIRDtc1
   ret

SendInfo:
   in areg,PINA
   rcall SendChar
   in areg,PINB
   rcall SendChar
   in areg,PIND
   rcall SendChar
   ret

SendCommand:
   ldi zh,high(usartdata)
   ldi zl,low(usartdata)
SendCommand0:
   ld areg,z+
   rcall SendChar
   dec breg
   brne SendCommand0
   ret

InitUSART:                     
   ldi areg,0
   out ubrrh,areg
   ldi areg,5                  
   out ubrrl,areg
   cbi ucsra,u2x
   ldi areg,(1 << RXCIE) | (1 << RXEN) | (1 << TXEN)
   out ucsrb,areg
   ldi areg,0b00000110
   out ucsrc,areg
   ret

SendChar:
   sbis ucsra,udre
   rjmp SendChar
   out udr,areg
   ret

SendCRLF:
   ldi areg,13
   rcall SendChar
   ldi areg,10
   rcall SendChar
   ret


Delay1ms:
   push areg
   push breg
   ldi breg,22

d1mslo1:
   wdr
   ldi areg,166
d1mslo0:
   dec areg
   brne d1mslo0
   dec breg
   brne d1mslo1

   pop breg
   pop areg
   ret

DelayXms:
   rcall Delay1ms
   dec areg
   brne DelayXms
   ret

Delay10ms:
   push areg
   ldi areg,10
   rcall DelayXms
   pop areg
   ret

Delay50ms:
   push areg
   ldi areg,50
   rcall DelayXms
   pop areg
   ret

Delay100ms:
   rcall Delay50ms
   rcall Delay50ms
   ret

Delay200ms:
   rcall Delay50ms
   rcall Delay50ms
   rcall Delay50ms
   rcall Delay50ms
   ret

version: .db "IR analyser/sender/repeater v1.0",0

SendVersion:
   ldi zh,high(version << 1)
   ldi zl,low(version << 1)
SendVersion0:
   lpm areg,z+
   tst areg
   breq SendVersion1
   rcall SendChar
   rjmp SendVersion0
SendVersion1:
   ret

 
Ds18SendData:
   lds   areg,ds18data+0
   rcall SendChar
   lds   areg,ds18data+1
   rcall SendChar
   lds   areg,ds18data+2
   rcall SendChar
   lds   areg,ds18data+3
   rcall SendChar
   lds   areg,ds18data+4
   rcall SendChar
   lds   areg,ds18data+5
   rcall SendChar
   lds   areg,ds18data+6
   rcall SendChar
   lds   areg,ds18data+7
   rcall SendChar
   lds   areg,ds18data+8
   rcall SendChar
   lds   areg,ds18err
   rcall SendChar
   ret

Delay5us:
   push   areg
   ldi    areg,15
Delay5us1:
   dec    areg
   brne   Delay5us1
   pop    areg
   ret

Delay15us:
   push  areg
   ldi   areg,51
Delay15us1:
   dec   areg
   brne  Delay15us1
   pop   areg
   ret

Delay60us:
   rcall Delay15us
   rcall Delay15us
   rcall Delay15us
   rcall Delay15us
   ret

Delay480us:
   push  breg
   ldi   breg,8
dsx18:
   rcall Delay60us
   dec   breg
   breq  dsx18e
   rjmp  dsx18  
dsx18e:
   pop   breg
   ret

Delay800ms:
   rcall Delay200ms
   rcall Delay200ms
   rcall Delay200ms
   rcall Delay200ms
   ret

OutDs18H:
   sbi   Por_DS18B20,Pix_DS18B20
   sbi   ddrb,Pix_DS18B20
   ret

OutDs18L:
   sbi   ddrb,Pix_DS18B20
   cbi   Por_DS18B20,Pix_DS18B20
   ret

InDs18:
   cbi   ddrb,Pix_DS18B20 
   cbi   Por_DS18B20,Pix_DS18B20
   nop
   sec
   sbis  Pin_DS18B20,Pix_DS18B20
   clc
   sbi   ddrb,Pix_DS18B20
   sbi   Por_DS18B20,Pix_DS18B20
   ret

ds18sendbyte:
   cli
   ldi   breg,8
ds18sendbyte1:
   rcall Delay5us
   rcall OutDs18L
   rcall Delay5us
   ror   areg
   brcc  ds18sendbyte2
   rcall OutDs18H
ds18sendbyte2:
   rcall Delay60us 
   rcall OutDs18H
   dec   breg
   breq  ds18sendbyteex
   rjmp  ds18sendbyte1
ds18sendbyteex:
   sei
   ret

ds18readbit:
   rcall Delay5us
   rcall OutDs18L
   rcall Delay5us
   rcall OutDs18H
   rcall InDs18
   ret

ds18readbyte:
   cli
   clr   areg
   ldi   breg,8
ds18readbyte1:
   rcall ds18readbit
   ror   areg
   rcall Delay60us
   dec   breg
   breq  ds18readbyteex
   rjmp  ds18readbyte1
ds18readbyteex:
   sei
   ret

ds18reset:
   rcall Delay5us
   rcall OutDs18L
   rcall Delay480us
   rcall OutDs18H
   rcall Delay15us
   rcall Delay60us
   rcall InDs18
   in    areg,sreg
   push  areg
   rcall Delay480us
   pop   areg
   out   sreg,areg
   ret

Ds18Measure:
   rcall ds18ClrError
   clr   areg
   sts   ds18crc,areg
   ldi   areg,0xFF
   sts   ds18data+0,areg
   sts   ds18data+1,areg
   sts   ds18data+2,areg
   sts   ds18data+3,areg
   sts   ds18data+4,areg
   sts   ds18data+5,areg
   sts   ds18data+6,areg
   sts   ds18data+7,areg
   sts   ds18data+8,areg
     
   rcall Ds18Measure1
   lds   areg,ds18err
   tst   areg
   brne  Ds18MeasureEx
   rcall Delay800ms
   rcall Ds18Measure2
Ds18MeasureEx:
   rcall OutDs18H
   ret

Ds18Measure1:
   rcall ds18reset
   brcs  Ds18error
   rcall ds18ClrError
   ldi   areg,0xCC
   rcall ds18sendbyte
   ldi   areg,0x44
   rcall ds18sendbyte
   ret

Ds18Measure2:
   rcall ds18ClrError
   rcall ds18reset
   brcs  Ds18error
   ldi   areg,0xCC
   rcall ds18sendbyte 
   ldi   areg,0xBE
   rcall ds18sendbyte
   rcall ds18readbyte 
   sts   ds18data+0,areg
   rcall ds18readbyte
   sts   ds18data+1,areg
   rcall ds18readbyte
   sts   ds18data+2,areg
   rcall ds18readbyte
   sts   ds18data+3,areg
   rcall ds18readbyte
   sts   ds18data+4,areg
   rcall ds18readbyte
   sts   ds18data+5,areg
   rcall ds18readbyte
   sts   ds18data+6,areg
   rcall ds18readbyte
   sts   ds18data+7,areg
   rcall ds18readbyte
   sts   ds18data+8,areg
;   rcall ds18reset
   ret

Ds18error:
   ldi   areg,1
   sts   ds18err,areg
   rcall OutDs18H
   ret

Ds18ClrError:
   clr   areg
   sts   ds18err,areg
   ret

Ds18WriteCfg:
   rcall ds18ClrError
   rcall ds18reset
   brcs  Ds18error
   ldi   areg,0xCC
   rcall ds18sendbyte 
   ldi   areg,0x4E
   rcall ds18sendbyte

   lds areg,usartdata+1
   rcall ds18sendbyte
   lds areg,usartdata+2
   rcall ds18sendbyte
   lds areg,usartdata+3
   rcall ds18sendbyte

   rcall ds18reset
   ret

Ds18ReadRom:
   rcall ds18ClrError
   rcall ds18reset
   brcs  Ds18error
   ldi   areg,0x33
   rcall ds18sendbyte

   rcall ds18readbyte
   sts   ds18data+0,areg
   rcall ds18readbyte
   sts   ds18data+1,areg
   rcall ds18readbyte
   sts   ds18data+2,areg
   rcall ds18readbyte
   sts   ds18data+3,areg
   rcall ds18readbyte
   sts   ds18data+4,areg
   rcall ds18readbyte
   sts   ds18data+5,areg
   rcall ds18readbyte
   sts   ds18data+6,areg
   rcall ds18readbyte
   sts   ds18data+7,areg
   clr   areg
   sts   ds18data+8,areg

   rcall ds18reset
   ret


WriteEprom:
   lds   areg,usartdata+1
   ldi   zh,high(usartdata+2)
   ldi   zl,low(usartdata+2)
   ldi   breg,8
WriteEprom1:
   push  breg
   ld    breg,z+
   rcall StoreEEPROM
   inc   areg
   pop   breg
   dec   breg
   brne  WriteEprom1
   ret

ReadEprom:
   ldi   breg,128
   ldi   areg,255
ReadEprom1:
   inc   areg
   push  areg
   rcall LoadEEPROM
   rcall SendChar
   pop   areg
   dec   breg
   brne  ReadEprom1
   ret

StoreEEPROM:
   sbic  eecr,eepe
   rjmp  StoreEEPROM
   cbi   eecr,EEPM1
   cbi   eecr,EEPM0
   out   eearl,areg 
   out   eedr,breg
   in    breg,sreg
   cli
   sbi   eecr,eempe
   sbi   eecr,eepe
   out   sreg,breg
   ret

LoadEEPROM:
   sbic  eecr,eepe
   rjmp  LoadEEPROM
   out   eearl,areg 
   sbi   eecr,eere
   in    areg,eedr
   ret






.exit


